package net.afyre.haha.demo.rcp.handlers;

import java.io.File;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.filesystem.EFS;
import org.eclipse.core.filesystem.IFileStore;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.handlers.HandlerUtil;
import org.eclipse.ui.ide.IDE;

/**
 * Handler for the 'open' command.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public class OpenHandler extends AbstractHandler {
  //private final FileHacks fileHacks = FileHacks.getInstance();
  
  /** {@inheritDoc} */
  @Override
  public Object execute(final ExecutionEvent event) throws ExecutionException {
    final Shell shell = HandlerUtil.getActiveShellChecked(event);
    final IWorkbenchWindow window = 
        HandlerUtil.getActiveWorkbenchWindowChecked(event);
    final IWorkbenchPage page = window.getActivePage();
    //final IWorkbench workbench = window.getWorkbench();
    final FileDialog dialog = new FileDialog(shell, SWT.OPEN);
    final String[] extensions = new String[]{ "*.haha" }; //$NON-NLS-1$
    final String[] filterNames = new String[] { 
        Messages.OpenHandler_HahaSourceFile 
    };
    dialog.setFilterExtensions(extensions);
    dialog.setFilterNames(filterNames);
    
    final String filename = dialog.open();
    if (filename != null) {
      final File file = new File(filename);
      if (file.exists() && file.isFile()) {
        final IFileStore fileStore = 
            EFS.getLocalFileSystem().getStore(file.toURI());
        try {
          IDE.openEditorOnFileStore( page, fileStore );
        } catch (final PartInitException e) {
          showError(shell, e);
        }
      }
    }
    return null;
  }
  
  private void showError(final Shell shell, 
                         final Throwable e) {
    final MessageBox dialog = new MessageBox(shell, SWT.ICON_ERROR|SWT.OK);
    dialog.setMessage(e.getLocalizedMessage());  
    dialog.open();
  }
}
