package net.afyre.haha.demo.solver.smt;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;

/**
 * A logger which prints all messages to a writer.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public class SmtWriterLogger implements SmtLogger {
  /** Output stream. */
  private final Writer writer;
  
  /**
   * Constructor.
   * @param writer Output stream.
   */
  public SmtWriterLogger(final Writer writer) {
    this.writer = writer;
  }
  
  /**
   * Writer a string, suppressing all I/O exceptions.
   * @param msg String to be written (can be null).
   */
  private void write(final String msg) {
    final String actualMessage;
    
    if (msg == null) {
      actualMessage = "null";
    } else {
      actualMessage = msg;
    }
    try {
      writer.write(actualMessage);
    } catch (final IOException e) {
      /* Ignore. */
      e.printStackTrace();
    }
  }
  
  /** {@inheritDoc} */
  @Override
  public void error(final String msg) { 
    write("ERROR: ");
    write(msg);
    write("\n");
  }

  /** {@inheritDoc} */
  @Override
  public void error(final String msg, 
                    final Throwable cause) { 
    write("ERROR: ");
    write(msg);
    write("\nCause:\n");
    cause.printStackTrace(new PrintWriter(writer));
    write("\n");
  }

  /** {@inheritDoc} */
  @Override
  public void model(final SmtModel model) {
    write("Counterexample: ");
    write(model.toString());
    write("\n");
  }
  
  /** {@inheritDoc} */
  @Override
  public void warning(final String msg) {
    write("WARNING: ");
    write(msg);
    write("\n");
  }

  /** {@inheritDoc} */
  @Override
  public void warning(final String msg, 
                      final Throwable cause) {
    write("WARNING: ");
    write(msg);
    write("\nCause:\n");
    cause.printStackTrace(new PrintWriter(writer));
    write("\n");
  }

  /** {@inheritDoc} */
  @Override
  public void info(final String msg) {
    write("INFO: ");
    write(msg);
    write("\n");
  }
  
  /** {@inheritDoc} */
  @Override
  public void input(final String text) {
    write(">>   ");
    write(text);
    write("\n");
  }
  
  /** {@inheritDoc} */
  @Override
  public void output(final String text) {
    write("<<   ");
    write(text);
    write("\n");
  }
}
