package net.afyre.haha.demo.solver.context;

import net.afyre.haha.demo.solver.expr.formulae.Formula;

/**
 * Declaration or definition of a propositional variable.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public final class PropDeclaration extends AbstractDeclaration {
  /** Value definition (null if the variable is free). */
  private final Formula value;

  /**
   * Create a variable declaration.
   * @param name Variable name.
   */
  public PropDeclaration(final String name) {
    super(name);
    this.value = null;
  }
  
  /**
   * Create a variable definition.
   * @param name Variable name.
   * @param value Value definition 
   *              (null if this is a free variable declaration).
   */
  public PropDeclaration(final String name,
                         final Formula value) {
    super(name);
    this.value = value;
  }

  /** {@inheritDoc} */
  @Override
  public void accept(final DeclarationVisitor visitor) {
    if (value == null) {
      visitor.visitPropDeclaration(getName());
    } else {
      visitor.visitPropDefinition(getName(), value);
    }
  }
  
}
