package net.afyre.haha.demo.solver.expr.formulae.impl;

import net.afyre.haha.demo.solver.expr.formulae.Formula;
import net.afyre.haha.demo.solver.expr.formulae.FormulaSwitch;
import net.afyre.haha.demo.solver.expr.terms.Term;
import net.afyre.haha.demo.solver.expr.types.ValueType;

/**
 * Local variable binding.
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public final class FLet extends AbstractFormula {
  /** Variable name. */
  private final String var;
  
  /** Variable type. */
  private final ValueType type;
  
  /** Value expression. */
  private final Term value;
  
  /** Expression body. */
  private final Formula body;

  /**
   * Constructor.
   * @param var Variable name.
   * @param type Variable type.
   * @param value Assigned value.
   * @param body Expression body.
   */
  public FLet(final String var, 
             final ValueType type,
             final Term value, 
             final Formula body) {
    this.var = var;
    this.type = type;
    this.value = value;
    this.body = body;
  }

  /**
   * Get the assigned variable.
   * @return Variable.
   */
  public String getVar() {
    return var;
  }

  /**
   * Get type of the assigned variable.
   * @return Type.
   */
  public ValueType getType() {
    return type;
  }
  
  /**
   * Get the assigned value.
   * @return Value.
   */
  public Term getValue() {
    return value;
  }

  /**
   * Get the expression body.
   * @return Expression.
   */
  public Formula getBody() {
    return body;
  }

  /** {@inheritDoc} */
  @Override
  public<Result> Result accept(final FormulaSwitch<? extends Result> v) {
    return v.caseFLet(var, type, value, body);
  }
}
