package net.afyre.haha.demo.solver.expr.formulae.impl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import net.afyre.haha.demo.solver.expr.Expression;
import net.afyre.haha.demo.solver.expr.VariableBinding;
import net.afyre.haha.demo.solver.expr.formulae.Formula;

/**
 * Abstract superclass of quantified formulae.
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public abstract class QuantifiedFormula extends AbstractFormula {
  /** List of bound variables. */
  private final List<VariableBinding> variables;
  
  /** Expression under quantification. */
  private final Formula formula;

  /** Instantiation patterns. */
  private final Expression[] patterns;
  
  /**
   * Constructor.
   * @param variables Bound variables.
   * @param formula Formula under quantification.
   * @param patterns Instantiation patterns.
   */
  protected QuantifiedFormula(final Iterable<VariableBinding> variables, 
                              final Formula formula,
                              final Expression... patterns) {
    this.variables = new ArrayList<VariableBinding>();
    for (final VariableBinding variable : variables) {
      this.variables.add(variable);
    }
    this.formula = formula;
    this.patterns = Arrays.copyOf(patterns, patterns.length);
  }

  /**
   * Get the variables bound by this quantifier.
   * @return Bound variables.
   */
  public Iterable<VariableBinding> getVariables() {
    return Collections.unmodifiableList(variables);
  }

  /**
   * Get the formula under quantification.
   * @return Formula.
   */
  public Formula getFormula() {
    return formula;
  }
  
  /**
   * Get the instantiation patterns.
   * @return Patterns.
   */
  public Expression[] getPatterns() {
    return Arrays.copyOf(patterns, patterns.length);
  }
}
