package net.afyre.haha.demo.solver.expr.terms.impl;

import java.util.Arrays;

import net.afyre.haha.demo.solver.expr.terms.Term;
import net.afyre.haha.demo.solver.expr.terms.TermSwitch;

/**
 * Function call.
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public final class Call extends AbstractTerm {
  /** Function name. */
  private final String function;
  
  /** Arguments. */
  private final Term[] arguments;
  
  /**
   * Constructor.
   * @param function Function name.
   * @param arguments Arguments.
   */
  public Call(final String function, final Term[] arguments) {
    this.function = function;
    this.arguments = Arrays.copyOf(arguments, arguments.length);
  }

  /**
   * Get the function name.
   * @return Function name.
   */
  public String getFunction() {
    return function;
  }

  /**
   * Get the arguments.
   * @return Arguments
   */
  public Term[] getArguments() {
    return Arrays.copyOf(arguments, arguments.length);
  }

  /** {@inheritDoc} */
  @Override
  public<Result> Result accept(final TermSwitch<? extends Result> v) {
    return v.caseCall(function, arguments);
  }
}
