package net.afyre.haha.demo.solver.expr.terms.impl;

import net.afyre.haha.demo.solver.expr.formulae.Formula;
import net.afyre.haha.demo.solver.expr.terms.Term;
import net.afyre.haha.demo.solver.expr.terms.TermSwitch;

/**
 * If-then-else construct.
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public final class IfThenElse extends AbstractTerm {
  /** Condition. */
  private final Formula condition;
  
  /** Value when the condition is true. */
  private final Term ifTrue;
 
  /** Value when the condition is false. */
  private final Term ifFalse;
  
  /**
   * Constructor.
   * @param condition Condition.
   * @param ifTrue Value when the condition is true.
   * @param ifFalse Value when the condition is false.
   */
  public IfThenElse(final Formula condition, 
              final Term ifTrue, 
              final Term ifFalse) {
    this.condition = condition;
    this.ifTrue = ifTrue;
    this.ifFalse = ifFalse;
  }

  /**
   * Get the condition.
   * @return Condition.
   */
  public Formula getCondition() {
    return condition;
  }

  /**
   * Get the value when the condition is true.
   * @return Value.
   */
  public Term getIfTrue() {
    return ifTrue;
  }

  /**
   * Get the value when the condition is false.
   * @return Value.
   */
  public Term getIfFalse() {
    return ifFalse;
  }

  /** {@inheritDoc} */
  @Override
  public<Result> Result accept(final TermSwitch<? extends Result> v) {
    return v.caseIfThenElse(condition, ifTrue, ifFalse);
  }
}
