package net.afyre.haha.demo.solver.expr.terms.impl;

import net.afyre.haha.demo.solver.expr.terms.Term;
import net.afyre.haha.demo.util.CmpUtil;
import net.afyre.haha.demo.util.HashUtil;

/**
 * Abstract superclass of operations which 
 * take a vector and an offset.
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public abstract class VectorOffsetOp extends AbstractTerm {
  /** Hash utilities. */
  private static final HashUtil hashUtil = HashUtil.getInstance();
  
  /** Comparison utilities. */
  private static final CmpUtil cmpUtil = CmpUtil.getInstance();

  /** Vector. */
  private final Term vector;
  
  /** Offset. */
  private final int offset;

  /** Cached hash code. */
  private final int hash;
  
  /**
   * Constructor.
   * @param vector Vector argument.
   * @param offset Offset argument.
   */
  public VectorOffsetOp(final Term vector, 
                        final int offset) {
    this.vector = vector;
    this.offset = offset;
    this.hash = hashUtil.hash(this.getClass().getName(), vector, offset);
  }

  /**
   * Get the vector argument.
   * @return the vector
   */
  public Term getVector() {
    return vector;
  }

  /**
   * Get the offset argument.
   * @return the offset
   */
  public int getOffset() {
    return offset;
  }
  
  /** {@inheritDoc} */
  @Override
  public int hashCode() {
    return hash;
  }
  
  /** {@inheritDoc} */
  @Override
  public boolean equals(final Object other) {
    if (other == null) {
      return false;
    } else if (this.getClass().equals(other.getClass())) {
      final VectorOffsetOp otherTerm = (VectorOffsetOp)other;
      return 
          cmpUtil.safeEquals(vector, otherTerm.vector) &&
            offset == otherTerm.offset;
    } else {
      return false;
    }
  }
}
