/**
 * 
 */
package net.afyre.haha.demo.solver.expr.types.impl;

import net.afyre.haha.demo.solver.expr.types.StructuredTypeSwitch;
import net.afyre.haha.demo.solver.expr.types.ValueType;

/**
 * Arrays.
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public final class Array extends AbstractStructuredType {
  /** Type of the indices. */
  private final ValueType indexType;
  
  /** Type of the elements. */
  private final ValueType elementType;

  /**
   * Constructor.
   * @param indexType Type of the indices.
   * @param elementType Type of the elements.
   */
  public Array(final ValueType indexType, 
               final ValueType elementType) {
    this.indexType = indexType;
    this.elementType = elementType;
  }

  /**
   * Get type of the indices.
   * @return Type.
   */
  public ValueType getIndexType() {
    return indexType;
  }

  /**
   * Get type of the elements.
   * @return Type.
   */
  public ValueType getElementType() {
    return elementType;
  }

  /** {@inheritDoc} */
  @Override
  public<Result> Result accept(final StructuredTypeSwitch<? extends Result> v) {
    return v.caseArray(indexType, elementType);
  } 
}
