package net.afyre.haha.demo.ui.handler;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.text.source.Annotation;
import org.eclipse.jface.text.source.IAnnotationModel;
import org.eclipse.jface.text.source.IAnnotationModelExtension;
import org.eclipse.ui.handlers.HandlerUtil;
import org.eclipse.ui.services.ISourceProviderService;
import org.eclipse.xtext.resource.XtextResource;
import org.eclipse.xtext.ui.editor.XtextEditor;
import org.eclipse.xtext.ui.editor.model.IXtextDocument;
import org.eclipse.xtext.util.concurrent.IUnitOfWork;

import net.afyre.haha.demo.ui.HahaSourceProvider;

/**
 * The common part of all the Haha handlers.
 * 
 * @author Aleksy Schubert <alx@mimuw.edu.pl>
 *
 */
public class HahaAbstractHandler extends AbstractHandler implements IHandler {

  /**
   * The default constructor that delegates construction to the superclass.
   */
  public HahaAbstractHandler() {
    super();
  }

  /**
   * The method clear error markers.
   * 
   * @param resource the resource in which the markers are cleared
   * @param annotationModel the model class that contains information on 
   *   location of markers
   */
  public void clearMarkers(final IResource resource,
                           final IAnnotationModel annotationModel) {
    if (resource != null) {
      try {
        resource.deleteMarkers("net.afyre.haha.demo.ui.problemmarker", true,
                               IResource.DEPTH_INFINITE);
      } catch (final CoreException e) {
        e.printStackTrace();
      }
    } else {
      final List<Annotation> annotations = new ArrayList<Annotation>();
      final Iterator<Annotation> it = annotationModel.getAnnotationIterator();
      while (it.hasNext()) {
        final Annotation annotation = it.next();
        if (annotation.getType().startsWith("net.afyre.haha")) {
          annotations.add(annotation);
        }
      }
      if (annotationModel instanceof IAnnotationModelExtension) {
        ((IAnnotationModelExtension) annotationModel)
            .replaceAnnotations(annotations.toArray(new Annotation[] {}), null);
      } else {
        for (final Annotation annotation : annotations) {
          annotationModel.removeAnnotation(annotation);
        }
      }
    }
  }

  /** {@inheritDoc} */
  @Override
  public Object execute(final ExecutionEvent event) throws ExecutionException {
    final Object editorObject = HandlerUtil.getVariable(event, "activeEditor");
    final XtextEditor editor = (XtextEditor) editorObject;
    final IXtextDocument doc = editor.getDocument();
    final IResource editorResource = editor.getResource();
    final IAnnotationModel annotationModel = editor.getDocumentProvider()
        .getAnnotationModel(editor.getEditorInput());
    final ISourceProviderService sourceProviderService = 
        (ISourceProviderService) HandlerUtil
          .getActiveWorkbenchWindow(event)
          .getService(ISourceProviderService.class);
    final HahaSourceProvider sourceProvider = 
        (HahaSourceProvider) sourceProviderService
          .getSourceProvider(HahaSourceProvider.VAR_RUNNING);
    final Thread thread = new Thread() {
      @Override
      public void run() {
        doc.readOnly(new IUnitOfWork<Object, XtextResource>() {
          public Object exec(final XtextResource resource) {
            clearMarkers(editorResource, annotationModel);
            return null;
          }
        });
      }
    };
    sourceProvider.setThread(thread);
    thread.start();

    return null;
  }
}
