package net.afyre.haha.demo.ui.highlighting;

import org.eclipse.swt.graphics.RGB;
import org.eclipse.xtext.ui.editor.syntaxcoloring.DefaultHighlightingConfiguration;
import org.eclipse.xtext.ui.editor.syntaxcoloring.IHighlightingConfigurationAcceptor;
import org.eclipse.xtext.ui.editor.utils.TextStyle;

/**
 * Defines text styles used by the highlighter.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public class HighlightingConfiguration 
  extends DefaultHighlightingConfiguration {

  /** Style used to color assertions (ID). */
  public final static String ASSERTION = "Assertion";
 
  /** Style used to color formula names (ID). */
  public final static String FORMULA_NAME = "FormulaName";
  

  /** {@inheritDoc} */
  @Override
  public void configure(final IHighlightingConfigurationAcceptor acceptor) {
    super.configure(acceptor);
    acceptor.acceptDefaultHighlighting(ASSERTION, "Assertions", 
                                       assertionStyle());
    acceptor.acceptDefaultHighlighting(FORMULA_NAME, "Formula names", 
                                      nameStyle());
  }
  
  /**
   * Create default style for assertions.
   * @return Assertion style.
   */
  private TextStyle assertionStyle() {
    TextStyle textStyle = new TextStyle();
    textStyle.setColor(new RGB(0, 0, 64));
    //textStyle.setStyle(SWT.ITALIC);
    return textStyle;
  }

  /**
   * Create default style for formula names.
   * @return Name style.  
   */
  private TextStyle nameStyle() {
    TextStyle textStyle = new TextStyle();
    textStyle.setColor(new RGB(127, 0, 0));
    return textStyle;
  }
}
