package net.afyre.haha.demo.ui.vcview;

import java.io.StringWriter;
import java.util.Iterator;

import net.afyre.haha.demo.solver.context.Declaration;
import net.afyre.haha.demo.vcgen.VCPrinter;
import net.afyre.haha.demo.vcgen.VerificationCondition;
import net.afyre.haha.demo.vcgen.tree.ConditionNode;
import net.afyre.haha.demo.vcgen.tree.ContextNode;
import net.afyre.haha.demo.vcgen.tree.Node;

import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.ViewPart;

import com.google.inject.Inject;
import com.google.inject.Singleton;

/**
 * View with table of verification conditions.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 *
 */
@Singleton
public class VerificationConditionsView extends ViewPart {
  public static final String ID = "net.afyre.haha.demo.ui.vcview";
  
  @Inject
  private VerificationConditionsSynchronizer sync;
  
  private TreeViewer viewer;
  
  private Text text;
    
  /** {@inheritDoc} */
  @Override
  public void createPartControl(final Composite parent) {
    final SashForm control = new SashForm(parent, SWT.HORIZONTAL);
    
    viewer = new TreeViewer(control);
    viewer.setContentProvider(new VerificationConditionsTree());
    viewer.setLabelProvider(new VCTreeLabelProvider());
    viewer.addSelectionChangedListener(new SelectionListener());
    
    text = new Text(control, SWT.MULTI|SWT.READ_ONLY);
    text.setFont(JFaceResources.getFont(JFaceResources.TEXT_FONT));
    
    control.setWeights(new int[] { 1, 1 });
  }

  /** {@inheritDoc} */
  @Override
  public void setFocus() {
    // TODO Auto-generated method stub
    
  }

  /** {@inheritDoc} */
  @Override
  public void init(final IViewSite site) throws PartInitException {
      super.init(site);
      sync.start(site);
  }

  /** {@inheritDoc} */
  @Override
  public void dispose() {
      sync.stop(getSite());
      super.dispose();
  }
  
  public void setTree(final Node root) {
    Display.getDefault().asyncExec(new Runnable() {
      public void run() {
        viewer.setInput(root);
        viewer.expandAll();
        updateText();
      }
    });
  }
  
  private void updateText() {
    final ITreeSelection selection = 
        (ITreeSelection) viewer.getSelection();  
    final StringWriter writer = new StringWriter();
    final VCPrinter printer = new VCPrinter(writer);
    
    final Iterator<?> it = selection.iterator();
    while (it.hasNext()) {
      final Object obj = it.next();
      if (obj instanceof ConditionNode) {
        final ConditionNode node = (ConditionNode)obj;
        VerificationCondition vc = node.getCondition();
        // HACK: Remove context from the condition.
        vc = new VerificationCondition(vc.getAssumptions(), 
                                       vc.getGoal(), null);
        printer.processVerificationCondition(vc);
      } else if (obj instanceof ContextNode) {
        for (final Declaration decl : 
                ((ContextNode)obj).getContext().getDeclarations()) {
          writer.write(decl.toString() + "\n");
        }
      }
    }
    final String result = writer.toString();
    text.setText(result);
  }
  
  private final class SelectionListener 
      implements ISelectionChangedListener {

    /** {@inheritDoc} */
    @Override
    public void selectionChanged(final SelectionChangedEvent event) {
      updateText();
    }
    
  }
}
