/**
 * 
 */
package net.afyre.haha.demo.util.tests;

import static org.junit.Assert.*;

import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * Utility methods used by tests.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public class Util {
  
  /**
   * Check if an iterator returns expected elements
   * and throws correct exception after iteration.
   * @param msg Error message (can be null).
   * @param it Iterator to test.
   * @param expected Expected elements.
   */
  public static void assertIteratorReturns(
      final String msg, 
      final Iterator<?> it, 
      final Object... expected) {
    int index = 0;
    final String pre = msg == null ? "" : (msg + ": ");
    
    assertNotNull("Null iterator", it);
    
    for (final Object expectedElement : expected) {
      assertTrue(
        String.format("%s: %d elements returned, %d expected", 
                      pre, index, expected.length), 
        it.hasNext());
      Object actualElement = null;
      
      try {
        actualElement = it.next();
      } catch (final NoSuchElementException e) {
        fail(String.format("%s: NSEE after %d elements, %d expected", 
                           pre, index, expected.length));
      }
      
      assertEquals(
        String.format("%s: Wrong element returned at index %d", pre, index),
        expectedElement, actualElement);
      index = index + 1;
    }
    assertFalse(
      String.format("%s: Too many elements returned (> %d)", 
                    pre, expected.length),
      it.hasNext());
    try {
      it.next();
    } catch (final NoSuchElementException e) {
      return;
    }
    fail(String.format("%s: NSEE not thrown after iteration", pre));
  }
  
  /**
   * Variant of {@link #assertIteratorReturns} with no message argument.
   * @param it Iterator to test.
   * @param expected Expected elements.
   */
  public static void assertIteratorReturns( 
      final Iterator<?> it, 
      final Object... expected) {
    assertIteratorReturns(null, it, expected);
  }

}
