package net.afyre.haha.demo.util;

/**
 * Immutable, comparable pair of elements
 *
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 *
 * @param <L> Type of the first component.
 * @param <R> Type of the second component.
 */
public class ComparablePair<
    L extends Comparable<? super L>, 
    R extends Comparable<? super R>> 
  extends Pair<L, R> 
  implements Comparable<ComparablePair<? extends L, ? extends R>> {

  /**
   * Constructor (internal use only).
   * @param first First element.
   * @param second Second element.
   */
  protected ComparablePair(final L first, final R second) {
    super(first, second);
  }

  /**
   * Create a new pair.
   * @param first First element.
   * @param second Second element.
   * @return Pair instance.
   */
  public static<
      L extends Comparable<? super L>, 
      R extends Comparable<? super R>>  ComparablePair<L, R> 
  getInstance(final L first, final R second) {
    return new ComparablePair<L, R>(first, second);
  }
  
  /** {@inheritDoc} */
  @Override
  public final int compareTo(
      final ComparablePair<? extends L, ? extends R> other) {
    if (other == null) {
      return 1;
    } else {
      return Pair.cmpUtil.compare(first, second, 
                                  other.first, other.second);
    }
  }  
}
