package net.afyre.haha.demo.types;

/**
 * A type switch which always returns the same value.
 * 
 * Subclasses may override this behaviour in selected cases.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 *
 * @param <Result> Type of values returned by this switch.
 */
public class TypeSwitchWithDefault<Result> implements TypeSwitch<Result> {
  /** Default value. */
  private final Result result;
  
  /**
   * Constructor (accessible to subclasses).
   * @param result Default result.
   */
  protected TypeSwitchWithDefault(final Result result) {
    this.result = result;
  }

  /** {@inheritDoc} */
  @Override
  public Result caseArray(final ValueType elementType) {
    return result;
  }

  /** {@inheritDoc} */
  @Override
  public Result caseZ() {
    return result;
  }

  /** {@inheritDoc} */
  @Override
  public Result caseInteger() {
    return result;
  }

  /** {@inheritDoc} */
  @Override
  public Result caseBoolean() {
    return result;
  }

  /** {@inheritDoc} */
  @Override
  public Result caseFunction(final ValueType resultType, 
                             final ValueType... argTypes) {
    return result;
  }
}
