package net.afyre.haha.demo.vcgen;

import net.afyre.haha.demo.solver.context.Answer;
import net.afyre.haha.demo.solver.context.AxiomDeclaration;
import net.afyre.haha.demo.solver.context.Declaration;
import net.afyre.haha.demo.solver.context.SolverContext;

// TODO: Better way of processing answers.

/**
 * Verification condition processor which uses a solver
 * to test program correctness.
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 *
 */
public final class VCSolver extends VCProcessor {
  /** Solver instance. */
  private final SolverContext solver;
  
  /** Last outcome. */
  private Answer lastAnswer;

  /**
   * Constructor.
   * @param solver Solver instance.
   */
  public VCSolver(final SolverContext solver) {
    this.solver = solver;
    lastAnswer = Answer.UNKNOWN;
  }
  
  /**
   * Get the outcome of last solver run.
   * @return Last answer.
   */
  public Answer getLastAnswer() {
    return lastAnswer;
  }

  /** {@inheritDoc} */
  @Override
  protected void processCondition(
      final VerificationCondition vc) {
    solver.mark();
    for (final DocumentedFormula pre : vc.getAssumptions()) {
      solver.addDeclaration(
        new AxiomDeclaration(pre.getName(), pre.getFormula()));
    }
    lastAnswer = solver.isValid(vc.getGoal().getFormula());
    solver.reset();
  }

  /** {@inheritDoc} */
  @Override
  protected void pushContext(final VCContext context) {
    if (context.containsDeclaration()) {
      solver.mark();
      for (final Declaration decl : context.getDeclarations()) {
        solver.addDeclaration(decl);
      }
    }
  }

  /** {@inheritDoc} */
  @Override
  protected void popContext(final VCContext context) {
    if (context.containsDeclaration()) {
      solver.reset();
    }
  }
  
  /** {@inheritDoc} */
  @Override
  public void end() {
    solver.finish();
  }
}
