package net.afyre.haha.demo.vcgen.tree;

import java.util.Collection;

/**
 * Node of a tree of verification conditions.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public interface Node {
  /**
   * Desconstruct this node with a switch.
   * @param <T> Result type.
   * @param sw Switch.
   * @return Value returned by the switch.
   */
  <T> T doSwitch(NodeSwitch<? extends T> sw);
  
  /**
   * Get the parent node, or {@code null} if this is the root node.
   * @return Parent node, {@code null} in the root node.
   */
  Node getParent();
  
  /**
   * Get child nodes.
   * @return List of nodes.
   */
  Collection<? extends Node> getChildren(); 
}
